#!/usr/bin/python
import re
def docstring2html(docstring):
	docstring = re.sub('``(.*?)(``|\'\')', '<code>\\1</code>', docstring)
	docstring = re.sub('`(.+?)(`|\')', '<code>\\1</code>', docstring)
	docstring = re.sub('\n:type\s+\w+\s*:\s*(.*?)(?=\n|$)', ' Type: <code>\\1</code>', docstring)
	docstring = re.sub('\n:param\s+(\w+)\s*:', '<br><b><code>\\1</code></b>:', docstring)
	docstring = re.sub(':rtype:', 'Return type: ', docstring)
	docstring = re.sub(':\w+:', '', docstring)
	docstring = docstring.replace("\n\n", "<br>")
	return docstring

def closing_parens(string, loc):
	escaped = False
	counter = 0
	string_char = None
	i = 0
	for c in string[loc:]:
		if counter == -1:
			break
		if string_char != None:
			if c == string_char and not escaped:
				string_char = None
			elif c == '\\':
				escaped = not escaped
			else:
				escaped = False
		elif c == '\'' or c == '"':
			string_char = c
		elif c == '(':
			counter += 1
		elif c == ')':
			counter -= 1
		i += 1
	if counter == -1:
		return loc+i-1
	return -1

from sys import argv, stdin
import json
import jedi
filename = argv[1]
line = int(argv[2])
column = int(argv[3])-1
source = stdin.read()
script = jedi.Script(source, line, column, filename)
completions = script.completions()
output = []

for completion in completions:
	item = {}
	kind = completion.type
	name = completion.name
	item["string"] = name
	description = completion.docstring(raw=False).strip()
	stripname = name
	stripcount = 0
	originaldescription = description
	match = re.match('(\w+\.)+(\w+)', description)
	if match:
		stripname = match.group(0)
	while description.startswith(stripname):
		description = description[len(stripname):].lstrip()
		stripcount += 1
		if description.startswith("("):
			cut = closing_parens(description, 1)
			if cut != -1:
				description = description[cut+1:].lstrip()
				if description.startswith("-> "):
					description = description[3:]
					newline = description.find("\n")
					if newline != -1:
						the_type = description[0:newline].lstrip()
						description = description[newline+1:]
						if not description.startswith("\n"):
							lendiff = len(originaldescription)-len(description)
							originaldescription = originaldescription[0:lendiff]+"\n"+originaldescription[lendiff:]
						description = description.lstrip()
						if not "type" in item:
							if the_type == "integer": the_type = "int"
							elif the_type.startswith("list of"): the_type = "list"
							elif the_type == "number" or the_type == "bool" or the_type == "value" or the_type == "string" or the_type == "character" or the_type == "object" or the_type == "list" or the_type == "dictionary" or the_type == "int" or the_type == "float" or the_type == "long" or the_type == "module" or the_type == "tuple" or the_type == "module":
								pass
							else: the_type = ""
							if the_type:
								item["type"] = the_type
	if stripcount == 0 and re.match('(\w+)(\.\w+)*(\(|$|\\n)', description):
		description = None
	elif stripcount > 1:
		description = originaldescription
	if description:
		item["description"] = docstring2html(description)
	if kind == "function" or kind == "class":
		try:
			insertion = ""
			separator = False
			optional = 0
			params = completion.params
			for param in params:
				name = param.description
				if len(name) == 0:
					continue
				search = name.find(" =")
				ellipsis = False
				if optional or search != -1:
					if search != -1:
						name = name[:search]
					if separator:
						insertion += " [, "
					else:
						insertion += "[ "
					optional += 1
				elif name == "...":
					ellipsis = True
					if separator:
						insertion += ", "
				elif separator:
					insertion += "#>, "
				if optional == 0 and separator and ellipsis == False:
					insertion += "<#"
				if separator == False:
					separator = True
				insertion += name
			if optional > 0:
				insertion += " "
			for x in range(0, optional):
				insertion += "]"
			if separator:
				insertion = "<#"+insertion+"#>"
			if separator or kind == "function":
				item["insertion"] = "("+insertion+")"
				kind = "function"
		except AttributeError:
			pass
	item["kind"] = kind
	output.append(item)
print(json.dumps(output))